-- Some xor-based Pan demos

module Xor where

import Int (fromInt)
import Monad
import Pan
import Simple (rippleUI,altRingsUI)
import CircleLimit (circleLimitUI)

hFlip, vFlip, hvFlip :: Filter c
hFlip  im = im . (\ (x,y) -> (-x, y))
vFlip  im = im . (\ (x,y) -> ( x,-y))
hvFlip im = im . (\ (x,y) -> (-x,-y))

xfFlipXor :: Filter BoolE -> Filter BoolE
xfFlipXor regXf reg = reg' `xorR` hvFlip reg'
 where
   reg' = regXf reg

shiftUI, orbitUI :: UI (Filter c)

-- Move to right
shiftUI = liftM (\ dx -> translate (dx,0))
                (slider "move" (-100,100) 50)

orbitUI = liftM2 (\ r ang -> translate (fromPolar (r,ang)))
                 (slider "radius" (0,100) 50)
                 fullAngleUI

wavePathUI = liftM (\ dx -> translate (dx, 100 * cos (dx/10)))
                   (slider "move" (0,100) 10)


shiftRingsUI = liftM2 xfFlipXor shiftUI altRingsUI

-- This one is not interesting.  it just rotates the whole picture
orbitRingsUI = liftM2 xfFlipXor orbitUI altRingsUI

wavePathRingsUI = liftM2 xfFlipXor wavePathUI altRingsUI

shiftRingsLimitUI =
  circleLimitUI `ap` liftM toImageC shiftRingsUI


-- Generalization of shiftRingsUI to n rings, distributed evenly at an
-- equal distance from the origin.
ringonNUI n = liftM2 (xorgonN n)
                (slider "separation" (0,300) 50)
                altRingsUI

xorgonN :: Int -> FloatE -> Region -> Region
xorgonN n r reg =
  xorRs [ translate (fromPolar (r,a)) reg
        | i <- [0 .. n-1]
        , let a = fromInt i * 2*pi / fromInt n
        ]

-- Equivalently:
xorgonN' n r reg = xorRs (map rf [0 .. n-1])
 where
  rf i = translate (fromPolar (r,a)) reg
   where a = fromInt i * 2*pi / fromInt n


xorgon :: Int -> IntE -> FloatE -> Region -> Region
xorgon maxN nE = map xorgonN [0 .. maxN] !!* nE
   

ringonUI maxN = liftM3 (xorgon maxN)
                  (islider "# rings" (1,maxN) 2)
                  (slider "separation" (-100,300) 50)
                  altRingsUI


xorRs :: [Region] -> Region
xorRs = foldr xorR (const false)

-------------------------

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileImageFor bends "xor rings shift" shiftRingsUI
  compileImageFor bends "xor rings wavy"  wavePathRingsUI
  --compileImageFor bends "xor rings shift limit" shiftRingsLimitUI
  compileImageFor bends "ringon to 12" $ ringonUI 12
  compileImageFor bends "ringon 20"    $ ringonNUI 20
