// ImageView.h: interface for the ImageView class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IMAGEVIEW_H__E436D0EC_9888_4244_9685_0C356F69D8A1__INCLUDED_)
#define AFX_IMAGEVIEW_H__E436D0EC_9888_4244_9685_0C356F69D8A1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PerfData.h"

// 
// If you want to enable the use of a second thread to 
// render a frame, uncomment the next line. (This is only
// advantageous if you're on a multi-processor system.)
#define USE_RENDER_THREAD

#import "PanObj.tlb" raw_interfaces_only no_namespace

//
// ImageView encapsulates a Pan image, associating the
// current viewing transform (zoom & translation) with one
// particular Pan image.
//
// An ImageView is capable of being displayed by a viewer window.
// 
class CImageView  
{
public:
    CImageView() : m_perfData(NULL), m_timeVarying(false),m_width(0),m_height(0) { };
	CImageView(CComQIPtr<IPanImage> disper, bool tVarying, CPerfData* pData);
	virtual ~CImageView();

    // void Paint(HWND hWnd, HDC hdc, RECT& rcPaint,
    //            int width, int height, int passNum, double maxJitter);

    // Render the image (using the current viewing transform). 
    void Render(RECT& rcPaint, int width, int height,
                int pass, double maxJitter);

    // Just BLiT the computed pixels onto the view.
    void Blit(HWND hWnd, HDC hdc, RECT& rcPaint, int width, int height);

    // Update generated image to reflect change in viewing transform, time
    // (if time varying.)
    void Redisplay();

    // choose to provide access to the viewing transform properties via
    // setters & getters.
    void SetZoom(double z) { m_zoom = z; m_image->SetViewTransform((float)m_panX,(float)m_panY,(float)m_zoom); }
    void SetViewSize(int w,int h) { if (m_image) m_image->SetViewSize(w,h); if (m_perfData) { m_perfData->SetViewSize(w,h); }}
    double GetZoom() { return m_zoom; }
    void SetTranslation(double x, double y) { m_panX = x; m_panY = y; m_image->SetViewTransform((float)m_panX,(float)m_panY,(float)m_zoom);}
    double GetPanX() { return m_panX; }
    double GetPanY() { return m_panY; }

protected:
    double m_zoom;
    double m_panX, m_panY;
    int m_width, m_height;
    int m_startMS;

    CComQIPtr<IPanImage> m_image;

	RGBQUAD* m_imageBuffer;
	bool m_timeVarying;

#ifdef USE_RENDER_THREAD
    HANDLE m_hRenderStart;
    HANDLE m_hRenderQuit;
    HANDLE m_hRenderComplete;
    HANDLE m_hThread;
    void*  m_rData;
#endif

    CPerfData* m_perfData;
};

#endif // !defined(AFX_IMAGEVIEW_H__E436D0EC_9888_4244_9685_0C356F69D8A1__INCLUDED_)
