// PanFileSelector.cpp: implementation of the CPanFileSelector class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <shlwapi.h>

#include "PanFileSelector.h"
#include <windowsx.h>



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define MAX_FILENAME_LENGTH 255

CPanImportImage::CPanImportImage( HWND              hWndParent,
                                  LPCTSTR           label,
                                  LPCTSTR           defVal,
                                  int               nWidth,
                                  int               nHeight,
                                  IntUpdater*       pUpd,
                                  void*             pParamBits,
                                  void*             pParamWidth,
                                  void*             pParamHeight,
                                  CInvalidator* pInvalid,
                                  CComBSTR          docURL)
   : m_pBits(0), m_fileName(NULL)
{
    USES_CONVERSION;

    m_updateValue       = pUpd;
    m_pParamBits        = pParamBits;
    m_pParamWidth       = pParamWidth;
    m_pParamHeight      = pParamHeight;
    m_iProc             = pInvalid;
    m_docURL            = docURL;

    Create(hWndParent, label, defVal, nWidth, nHeight, -1);

    m_fileName = new TCHAR[MAX_FILENAME_LENGTH];

    _tcscpy(m_fileName,defVal);

    LoadImage(T2A(m_fileName));

}

CPanImportImage::~CPanImportImage()
{
    if (m_pBits != 0) {
	    free((void*)m_pBits);
    }

    delete [] m_fileName;
}

void
CPanImportImage::LoadImage(char* fileName)
{
    USES_CONVERSION;
    int theBits;
    int width=0, height=0;
    theBits = LoadImportedImg(fileName, &width, &height);

    if (theBits == 0) {
       // it might just be a URL....
       TCHAR effectURL[MAX_PATH];
       TCHAR cachedURL[MAX_PATH];
       DWORD dwLen = MAX_PATH;

       // requires at least Win98 (or Win95 with IE5)
       BOOL isValidURL = UrlIs(m_fileName,URLIS_URL);

       if (isValidURL && SUCCEEDED(URLDownloadToCacheFile(NULL, m_fileName, cachedURL, dwLen, 0, NULL)) ) {
           // try using the absolute URL
           theBits = LoadImportedImg(T2A(cachedURL), &width, &height);
       } else if ( SUCCEEDED(UrlCombine(OLE2T(m_docURL), m_fileName, effectURL, &dwLen,0)) ) {
           // last-chance saloon, if the Effect is a relative URL, try combining
           // it with the URL of the document the control appears in & try to load that.
           dwLen = MAX_PATH;
           if (SUCCEEDED(URLDownloadToCacheFile(NULL, effectURL, cachedURL, dwLen, 0, NULL))) {
               // Try opening the cached file...
               theBits = LoadImportedImg(T2A(cachedURL),&width,&height);
           }
        }
    }

	if (theBits != 0) {

        // let go of previous image.
	    if (m_pBits != 0) {
	        free((void*)m_pBits);
	        m_pBits = 0;
	    }

        m_pBits = theBits;

        m_updateValue->UpdateInt(m_pParamWidth, width);
        m_updateValue->UpdateInt(m_pParamHeight, height);
        m_updateValue->UpdateInt(m_pParamBits, m_pBits);
	    m_iProc->Invalidate();
	}
}


LRESULT
CPanImportImage::OnChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    USES_CONVERSION;

    if (m_fileName != NULL) { 
        ::Edit_GetText(hWndCtl, m_fileName, MAX_FILENAME_LENGTH-1);
        LoadImage(T2A(m_fileName));
    }
	return 0;   
}

LPCTSTR CPanImportImage::GetString()
{
    return m_fileName;
}

void CPanImportImage::SetString(LPCTSTR str)
{
    _tcsncpy(m_fileName,str,MAX_FILENAME_LENGTH-1);
    SetFileName(m_fileName);
}
