// Slider.cpp: implementation of the CSlider class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Slider.h"
#include <windowsx.h>
#include <tchar.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSlider::CSlider()
{
    m_isVertical = false;
}

CSlider::~CSlider()
{
    delete m_tracker;
    delete m_edit;
}


HWND
CSlider::Create
		( HWND     hWndParent
		, LPCTSTR  label
		, LPCTSTR  initEditVal
		, int nWidth, int nHeight
		)
{
    HWND  hWndTracker, hWndEdit;
    RECT r;
    SIZE* pSize;

    r.top = 0;
    r.left = 0;
    r.bottom = nHeight;
    r.right = nWidth;

    CBox::Create(label, 150,100, hWndParent);

    m_tracker = new CTrackBar();
    hWndTracker = m_tracker->Create(*this, r, NULL,0,0,ID_TRACKBAR);
    m_tracker->SetRange(SLIDER_MIN_RANGE, SLIDER_MAX_RANGE);

    r.top = 0;
    r.left = 0;
    r.bottom = 20;
    r.right = 30;

    m_edit = new CEditBox(ES_RIGHT);
    hWndEdit = m_edit->Create(*this, r, initEditVal, 0,0, ID_EDIT_FIELD);

    // indicate that neither the edit control nor the trackbar should
    // be resized vertically.
    SetProp(hWndTracker, RESIZE_PROPERTY_VERT,  (HANDLE)RESIZE_NONE);
    SetProp(hWndEdit,    RESIZE_PROPERTY_VERT,  (HANDLE)RESIZE_NONE);
    SetProp(hWndTracker, RESIZE_PROPERTY_HORIZ, (HANDLE)RESIZE_NORMAL);
  
    //
    // Store original sizes so that we can resize correctly later on.
    //
    pSize = (SIZE*)malloc(sizeof(SIZE));
    if (pSize) {
      RECT r;

      ::GetWindowRect(hWndTracker, &r);

      pSize->cx = r.right - r.left;
      pSize->cy = r.bottom - r.top;
      SetProp(hWndTracker, RESIZE_PROPERTY_ORIGSIZE, (HANDLE)pSize);
    }

    // Finished adding components, lay them out.
    CalcLayout();

    return m_hWnd;
}


LRESULT CSlider::OnHScroll(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    DWORD ev = LOWORD(wParam);
      
	if ( ev == TB_ENDTRACK || ev == TB_THUMBTRACK || ev == TB_THUMBPOSITION) {
	  TCHAR msg[80];
	  int val;
	
	  val = SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
	  _stprintf(msg, TEXT("%d"), val);
	  // update the edit field with new value.
      ::Edit_SetText(GetDlgItem(ID_EDIT_FIELD), msg); 
    } 

    return 0;

}

LRESULT CSlider::OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    HWND hWndChild;

	RemoveProp(m_hWnd, RESIZE_PROPERTY_VERT);
	RemoveProp(m_hWnd, RESIZE_PROPERTY_HORIZ);
	hWndChild = GetWindow(GW_HWNDFIRST);

	while (hWndChild) {
	  RemoveProp(hWndChild,RESIZE_PROPERTY_VERT);
	  RemoveProp(hWndChild,RESIZE_PROPERTY_HORIZ);
      hWndChild = ::GetWindow(hWndChild, GW_HWNDNEXT);
	}

    return 0;
}
