
//
// Prototypes and defintions needed by the Pan generated
// code (all backends.)
// 
#ifndef __PANPLUGIN_H__
#define __PANPLUGIN_H__

#include "PanTrig.h"
#ifdef WANT_FP_SIMD
#include <fvec.h>
#endif
#include "PanDisp.h"

extern void InvalidateView(void*);
extern "C" int LoadImportedImg(char* fName, int* w, int* h);

#define ALLOC_ARRAY(v,ty,sz) v = (ty)_alloca (sz)

#if 0
#define ALLOC_ARRAY(v,ty,sz) \
   if (v == NULL) { \
      v = (ty)_alloca (sz);\
   } else { \
      ; \
   }
#endif

#define ALLOC_HEAP_ARRAY(v,ty,isArrElts,sz) \
     v = (ty)malloc(sz + 2*sizeof(int)); \
     *(int*)v = isArrElts; \
     *((int*)v+1) = sz; \
     v = (ty)((int*)v+2)

//typedef void (*PhSDisplayer)(int width, int height, int output, int outPad, int pProc, int doProgress, int pParams);

//
// filtering displayer - receives an input image (of given depth),
// and produces an output image. Used by video processing and image
// transformers. A slightly different version is used for PhotoShop
// plugins; see below.
//
typedef void (*FilterDisplayer) 
                ( float t      // elapsed time
				, float panX    // displacementX
				, float panY    //   "         Y
				, float zoom    // zoom factor
				, int width     // image width
				, int height    // image height
				, int imgWidth  // image width
				, int imgHeight // image height
				, int input     // image data
				, int  cMap     // colour map
				, int output    // [out] pointer to where to deposit the gen'ed image.
				);


// Some operators missing from fvec.h

// Of dubious value, because of inexactness.
#if WANT_FP_SIMD
bool operator ==(F32vec4 a, F32vec4 b)
{
    // Is there should be a faster way, using 
    F32vec4 c = a - b;
    return c[3] == 0 && c[2] == 0 && c[1] == 0 && c[0] == 0;
}

F32vec4 operator -(F32vec4 a)
{
    return F32vec4(-a[3],-a[2],-a[1],-a[0]);
}
#endif

// Are these guys in some standard place?
template<class T> inline T pan_min(T a, T b) {return a<b?a:b;}
template<class T> inline T pan_max(T a, T b) {return a<b?b:a;}

// What does the standard "abs" do?  It seems to produce an integer.
template<class T> inline T pan_abs(T a) {return a>=0?a:-a;}

// template<class T> inline T pan_ipow(T a, int n)
// { return pow(a, (double) n); }

// Do as loop instead, counting on the C compiler to unroll.  Note that
// normal form guarantees.  Also, handle positive case here and general
// below.  Note avoidance of recursion so we can inline fully.
// ToDo: use log-time binary algorithm, rather than simple linear one.

// Still used?:
template<class T> inline T pan_ipowPos(T a, int n)
{
    // At some point it's cheaper to call general the pow function.  What point?
    if (n > 20)
        return pow(a, (double) n);
    else {
        T prod = 1; int i=n;
        while (i) { prod*=a; i--; }
        return prod;
    }
}

template<class T> inline T pan_ipow(T a, int n)
{
    return (n < 0) ? (1 / pan_ipowPos(a,-n)) : (pan_ipowPos(a,n));
}

// I hope the C compiler will clean up the cse that base is a known constant.
inline double pan_logBase(double base, double val)
{
    return log(val) / log(base);
}

template<class A,class B>
struct Pair {
    Pair(A a, B b) : fst(a), snd(b) {}
    A fst; B snd;
};

template<class A, class B> inline Pair<A,B> pan_tuple(A a,B b) { return Pair(a,b); }
template<class A, class B> inline A pan_elt0(Pair<A,B> p) { return p.fst; }
template<class A, class B> inline B pan_elt1(Pair<A,B> p) { return p.snd; }

// More overloadings of pan_elt<i> can go here


//
// Unfortunately, using /QIfist to provide a fast impl. of F->I fails for
// (currently) unknown reasons. It seems to interact badly with /Ox in an example
// like: disp "t" $ do { i1 <- imageSource "1" ; return $ cond xPos i1 blackI }
// If you turn off /Ox but leave on /QIfist, it produces correct code, but
// enabling them both introduces some semi-random artifacts in the generated image.
// 
// So, instead of using /QIfist, we provide a cheap as possible implementation of
// _ftol(), which rounds towards minus inf. (assuming that's the rounding mode
// currently set - which it is.) We do pay the cost of a function call,
// but at least it isn't as expensive as the round-towards-zero implementation
// of _ftol() provided by VC++.
//
// ToDo: investigate why /Ox and /QIfist seems to produce wrong code.
// 
#if defined(_X86_) && !defined(NO_FTOL_FOR_ME)
extern "C" {
__declspec(naked) int _cdecl _ftol(double f) {
 __asm {
   lea   ecx, [esp-4]
   sub   esp,4
   fistp dword ptr [ecx]
   mov   eax, [ecx]
   add   esp,4
   ret 0
     }
}
}
#endif

// A PhotoShop plugin can callback to the host to do progress updates,
// ProgressProc is the type of this callback.
typedef void (*ProgressProc) (int done, int total);

#endif /* __PANPLUGIN_H__ */
