/*
 * Misc. utilities for Pan plugins.
 *
 */
#include "PIFilter.h"
#include "PIUtilities.h"
#include "plugin.h"
#include "utils.h"
#include <stdio.h>

Boolean
setRect
     ( Rect* pRect
     , short nLeft
     , short nTop
     , short nRight
     , short nBottom
     )
{
  if (!pRect) {
    return FALSE;
  }
    
  pRect->left   = nLeft;
  pRect->top    = nTop;
  pRect->right  = nRight;
  pRect->bottom = nBottom;
  
  return TRUE;
}

/*****************************************************************************/

void
scaleRect(Rect *l, short n, short d)
{
  l->left = (l->left * n) / d;
  l->top = (l->top * n) / d;
  l->right = (l->right * n) / d;
  l->bottom = (l->bottom * n) / d;
}

void
scaleRectY(Rect *l, short n, short d)
{
  l->top = (l->top * n) / d;
  l->bottom = (l->bottom * n) / d;
}

void
scaleRectX(Rect *l, short n, short d)
{
  l->left = (l->left * n) / d;
  l->right = (l->right * n) / d;
}

void
shrinkRect(Rect *l, short x, short y)
{
  l->left += x;
  l->top += y;
  l->right -= x;
  l->bottom -= y;
}

void
copyRect(Rect *l, const Rect *r)
{
  l->left = r->left;
  l->top = r->top;
  l->right = r->right;
  l->bottom = r->bottom;
}

/***********************************************************************************/

/* Computes the scaled down rectangle and the scale factor for the proxy */

void
CalcProxyScaleFactor
          ( const Rect *filterRect
	  , Rect *proxyRect
	  , short *scaleFactor
	  )
{

  short filterHeight;
  short filterWidth;
  short itemHeight;
  short itemWidth;
  Point fraction;

  //  shrinkRect (proxyRect, kGutter, kGutter); 		 // for the border
  
  filterHeight = filterRect->bottom - filterRect->top;
  filterWidth  = filterRect->right  - filterRect->left;
	
  itemHeight = proxyRect->bottom - proxyRect->top;
  itemWidth  = proxyRect->right  - proxyRect->left;
	
  if (itemHeight > filterHeight)
    itemHeight = filterHeight;
		
  if (itemWidth > filterWidth)
    itemWidth = filterWidth;
	
  fraction.h = (short)((filterWidth + itemWidth) / itemWidth);
  fraction.v = (short)((filterHeight + itemHeight) / itemHeight);

  if (fraction.h > fraction.v) 
    *scaleFactor = fraction.h;
  else
    *scaleFactor = fraction.v;

  copyRect (proxyRect, filterRect);	
  scaleRectX(proxyRect, 1 , fraction.h);
  scaleRectY(proxyRect, 1 , fraction.v);
  //  scaleRect (proxyRect, 1, *scaleFactor);
}

//
//   FUNCTION: LoadBMPBitsPal(char*, void*, void*);
//
//   PURPOSE: Load the palette and bits of a BMP.
//
//   COMMENTS:
//
//        Given the name of a BMP file, tries to look at its header,
//        returning the attributes of interest to Pan.
//
BOOL
LoadBMPBitsPal
         ( /*[in]*/   char* lpszFile
	 , /*[out]*/  RGBQUAD** pBits
	 , /*[out]*/  RGBQUAD** pPalette
	 )
{
  HANDLE h;
  BOOL res;
  BITMAPFILEHEADER bmf;
  DWORD bytesRead = 0;
  BITMAPINFOHEADER bih;
  DWORD size;
  DWORD offset;
  DWORD palSizeBytes;
  RGBQUAD* palette;

  h = CreateFile( lpszFile
  	        , GENERIC_READ 
		, FILE_SHARE_READ
		, NULL
		, OPEN_EXISTING
		, FILE_ATTRIBUTE_NORMAL
		, NULL
		);

  if ( INVALID_HANDLE_VALUE == h ) {
     return FALSE;
  }

  res = ReadFile(h, &bmf, sizeof(BITMAPFILEHEADER), &bytesRead, NULL);
  
  // Check whether the file header is reasonable before continuing:
  if ( FALSE == res || bytesRead != sizeof(BITMAPFILEHEADER) || 
       bmf.bfType != (WORD)((int)'B' + 256* (int)'M') ||
       bmf.bfReserved1 != 0 || bmf.bfReserved2 != 0 ) {
     CloseHandle(h);
     return FALSE;
  }
  
  size   = bmf.bfSize - sizeof(BITMAPFILEHEADER);
  offset = bmf.bfOffBits - sizeof(BITMAPFILEHEADER);
  
  res = ReadFile(h, &bih, sizeof(BITMAPINFOHEADER), &bytesRead, NULL);
  
  palSizeBytes = sizeof(RGBQUAD)*bih.biClrUsed;

  palette = (RGBQUAD*)malloc(palSizeBytes);
  
  if ( NULL == palette ) {
     CloseHandle(h);
     return FALSE;     
  }
  
  res = ReadFile(h, palette, palSizeBytes, &bytesRead, NULL);
  
  if ( FALSE == res || bytesRead != palSizeBytes ) {
	CloseHandle(h);
        free(palette);
	return FALSE;  
  }
  
  *pBits = (RGBQUAD*)malloc(size);
  
  if ( NULL == *pBits ) {
     CloseHandle(h);
     free(palette);
     return FALSE;     
  }

  res = ReadFile(h, *pBits, size, &bytesRead, NULL);

  if (  FALSE == res ) {
     CloseHandle(h);
     free(palette);
     free(*pBits);
     return FALSE;
  }
  
  *pPalette = (RGBQUAD*)palette;

  // Close down and return back home.
  CloseHandle(h);
  return TRUE;
}
