%
% (c) @CopyrightHolder@, @CopyrightPeriod@
%

Ragged band of commonly used pretty-printing combinators.

\begin{code}
module PrettyUtils 
	( vsepWith    -- :: Doc -> [Doc] -> Doc 
	, hsepWith    -- :: Doc -> [Doc] -> Doc 
	, vsep        -- :: [Doc] -> Doc

	, ppDecls      -- :: [Doc] -> Doc
	, ppTuple      -- :: [Doc] -> Doc
	, ppTupleVert  -- :: [Doc] -> Doc
	, ppList       -- :: [Doc] -> Doc
	, ppListVert   -- :: [Doc] -> Doc
	) where

import Pretty
import List ( intersperse )
\end{code}

\begin{code}
-- vsepWith comma [x1,x2] ==>  x1 <> comma $$ x2 <> comma
vsepWith :: Doc -> [Doc] -> Doc
vsepWith _ [] = empty
vsepWith _ [x] = x
vsepWith d (x:xs) = x <> d $$ vsepWith d xs

hsepWith :: Doc -> [Doc] -> Doc
hsepWith _ [] = empty
hsepWith d (x:xs) = x <> d <+> hsepWith d xs

vsep :: [Doc] -> Doc
vsep [] = empty
vsep ds = foldr1 ($+$) ds

ppDecls :: [Doc] -> Doc
ppDecls [] = empty
ppDecls ls = vsepWith semi ls <> semi

ppTuple :: [Doc] -> Doc
ppTuple ls = parens $ hcat (intersperse comma ls)

ppList :: [Doc] -> Doc
ppList ls = char '[' <+> hcat (intersperse comma ls) <+> char ']'

-- ppTupleVert [x1,x2,x3] ==>
--       ( x1
--       , x2
--       , x3
--       )
ppTupleVert :: [Doc] -> Doc
ppTupleVert []     = parens empty
ppTupleVert (a:as) = vsep (char '(' <+> a : map ((<+>) comma) as) $$ char ')'

ppListVert :: [Doc] -> Doc
ppListVert []     = text "[]"
ppListVert (a:as) = vsep (char '[' <+> a : map ((<+>) comma) as) $$ char ']'
\end{code}
